import unittest
import time
import json

from pylognorm import LogNormalizer, lib_version


APACHE_LL = (
    '66.69.25.244 - - [13/Jan/2013:09:57:51 -0600] "GET /wiki/skins/common/'
    'commonPrint.css?270 HTTP/1.1" 304 212 "http://nu.realityhub.com/wiki/i'
    'ndex.php/Main_Page" "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:18.0) Geck'
    'o/20100101 Firefox/18.0"'
)


class WhenUsingBindings(unittest.TestCase):

    def test_init(self):
        normalizer = LogNormalizer()

    def test_version(self):
        version = lib_version()
        self.assertIsNotNone(version)
        self.assertTrue(version != '')

    def test_loading_rules(self):
        normalizer = LogNormalizer()
        normalizer.load_rules('./samples/rules.db')

    def test_normalizing(self):
        normalizer = LogNormalizer()
        normalizer.load_rules('./samples/rules.db')
        normalized_ll = normalizer.normalize(APACHE_LL)

        expected = {
            'user_agent': 'Mozilla/5.0 (Windows NT 6.1; WOW64;'
                          ' rv:18.0) Gecko/20100101 Firefox/18.0',
            'url': 'http://nu.realityhub.com/wiki/index.php/Main_Page',
            'bytes': '212',
            'status_code': '304',
            'http_version': 'HTTP/1.1',
            'uri': '/wiki/skins/common/commonPrint.css?270',
            'method': 'GET',
            'timestamp': '13/Jan/2013:09:57:51 -0600',
            'b': '-',
            'a': '-',
            'remote_host': '66.69.25.244'
        }

        self.assertEqual(expected, json.loads(normalized_ll))

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
try:
    from setuptools import setup, find_packages
    from setuptools.command import easy_install
except ImportError:
    from ez_setup import use_setuptools
    use_setuptools()
    from setuptools import setup, find_packages
    from setuptools.command import easy_install

try:
    from Cython.Compiler.Main import compile
    from Cython.Distutils import build_ext
    has_cython = True
except ImportError:
    has_cython = False

import os
from distutils.core import setup
from distutils.extension import Extension

C_LIBRARIES = ['estr', 'ee', 'lognorm']
COMPILER_ARGS = list()
LINKER_ARGS = list()

ENABLE_GDB = os.getenv('ENABLE_GDB')

if ENABLE_GDB and ENABLE_GDB.lower() == 'true':
    COMPILER_ARGS.append('-g')
    LINKER_ARGS.append('-g')

cmdclass = dict()
ext_modules = list()


def read(relative):
    try:
        contents = open(relative, 'r').read()
        return [l for l in contents.split('\n') if l != '']
    except Exception:
        print('Unable to read from build file: {}'.format(relative))
        return list()


def ez_install(package):
    easy_install.main(["-U", package])


def module_files(module_name, *extensions):
    found = list()
    filename_base = module_name.replace('.', '/')
    for extension in extensions:
        filename = '{}.{}'.format(filename_base, extension)
        if os.path.exists(filename):
            found.append(filename)
            break
    return found


def cythonize():
    if has_cython:
        cmdclass.update({
            'build_ext': build_ext
        })

    for module in read('./tools/cython-modules'):
        if has_cython:
            build_list = module_files(module, 'pyx', 'pyd')
            for build_target in build_list:
                compile(build_target)
        else:
            build_list = module_files(module, 'c')

        ext_modules.append(
            Extension(
                module,
                build_list,
                libraries=C_LIBRARIES,
                extra_compile_args=COMPILER_ARGS,
                extra_link_args=LINKER_ARGS))


cythonize()

setup(
    name='pylognorm',
    version='0.1.3',
    description='liblognorm python bindings',
    url='http://github.com/zinic/pylognorm',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Natural Language :: English',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Topic :: System :: Logging',
        'Programming Language :: Python',
        'Programming Language :: Cython',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3'
    ],
    author='John Hopper',
    author_email='john.hopper@jpserver.net',
    license='Apache 2.0',
    tests_require=read('./tools/test-requires'),
    install_requires=read('./tools/install-requires'),
    test_suite='nose.collector',
    zip_safe=False,
    include_package_data=True,
    packages=find_packages(exclude=['ez_setup']),
    cmdclass=cmdclass,
    ext_modules=ext_modules
)



from os import path
from liblognorm cimport *
from libc.stdlib cimport malloc, free
from libc.string cimport strlen


def lib_version():
    cdef char* version = ln_version()
    return PyString_FromStringAndSize(version, strlen(version))


cdef char * _object_to_cstr(object obj):
    cdef char *cstr_filename
    if PyByteArray_Check(obj):
        cstr_filename = PyByteArray_AsString(obj)
    elif PyBytes_Check(obj):
        cstr_filename = PyBytes_AsString(obj)
    else:
        raise Exception(
            'Unable to convert to cstr: {}'.format(type(obj)))
    return cstr_filename


class LogNormalizer(object):

    def __init__(self):
        self.cnormalizer = CLogNormalizer()
        self.rules_loaded = False

    def load_rules(self, filename):
        if self.rules_loaded:
            raise Exception('Normalizer rules already loaded.')
        if not path.exists(filename):
            raise Exception('Unable to locate file: {}'.format(filename))
        self.cnormalizer.load_rules(filename)
        self.rules_loaded = True

    def normalize(self, logline):
        return self.cnormalizer.normalize(logline, len(logline))


cdef class CLogNormalizer(object):

    cdef ln_ctx normalizer_ctx
    cdef ee_ctx cee_ctx

    def __cinit__(self):
        self.normalizer_ctx = ln_initCtx()
        self.cee_ctx = ee_initCtx()
        ln_setEECtx(self.normalizer_ctx, self.cee_ctx);

    def __init__(self):
        pass

    def __dealoc__(self):
        ee_exitCtx(self.cee_ctx)
        ln_exitCtx(self.normalizer_ctx)

    def load_rules(self, object filename):
        cdef char* cstr_filename

        if PyByteArray_Check(filename):
            cstr_filename = PyByteArray_AsString(filename)
        elif PyBytes_Check(filename):
            cstr_filename = PyBytes_AsString(filename)
        else:
            raise Exception(
                'Unable to convert to cstr: {}'.format(type(filename)))

        ln_loadSamples(self.normalizer_ctx, cstr_filename)

    def normalize(self, object data, int length):
        cdef char *normal = self._normalize(_object_to_cstr(data), length)
        if normal == NULL:
            return None
        else:
            pyobj = PyUnicode_FromString(normal)
            free(normal)
            return pyobj

    cdef char * _normalize(self, char *data, int length):
        cdef es_str_t *in_str = es_newStrFromCStr(data, length)
        cdef ee_event *event = NULL
        cdef char *formatted = NULL

        ln_normalize(self.normalizer_ctx, in_str, &event)

        if event != NULL:
            formatted = self.format_JSON(event)
            ee_deleteEvent(event)
        es_deleteStr(in_str)
        return formatted

    cdef char * format_JSON(self, ee_event *event):
        cdef es_str_t *out_str = NULL
        cdef char *cstr = NULL

        ee_fmtEventToJSON(event, &out_str)
        cstr = es_str2cstr(out_str, NULL)
        es_deleteStr(out_str)
        return cstr

